/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.

 Christian Herdtweck, Intra2net AG 2015
 */

#include "dns/resolverbase.h"
#include "dns/dnsmaster.h"

#include <boost/bind.hpp>


ResolverBase::~ResolverBase()
{ }

/**
 * callbacks should be of type
 * void resolve_callback(const bool was_success,
 *                       const int cname_count)
 */
void ResolverBase::async_resolve(const callback_type &callback)
{
    // remember callback
    CallbackList.push(callback);

    // let subclass do the resolving
    do_resolve();
}

ResolverBase::ResolverBase(const IoServiceItem &io_serv,
             const std::string &hostname,
             const DnsCacheItem &cache )
    : IoService( io_serv )
    , Hostname( hostname )
    , Cache( cache )
    , CallbackList()
{}

void ResolverBase::update_cache( const std::string &hostname,
                                 const HostAddressVec &new_results ) const
{   Cache->update( hostname, new_results );  }

void ResolverBase::update_cache( const std::string &hostname,
                                 const std::string &cname ) const
{   Cache->update( hostname, cname );  }

void ResolverBase::update_cache( const HostAddressVec &new_results ) const
{   Cache->update( Hostname, new_results );  }

void ResolverBase::update_cache( const std::string &cname ) const
{   Cache->update( Hostname, cname );  }

void ResolverBase::update_cache_ttl( const uint32_t ttl ) const
{   Cache->update_ttl( Hostname, ttl );  }

HostAddressVec& ResolverBase::get_cached_ips_recursively(const std::string host)
                                                                           const
{
    if (host.empty())
        return Cache->get_ips_recursive(Hostname);
    else
        return Cache->get_ips_recursive(host);
}

void ResolverBase::schedule_callbacks(const bool was_success,
                                      const int cname_count)
{
    while ( !CallbackList.empty() )
    {
        callback_type callback = CallbackList.front();
        CallbackList.pop();
        IoService->post( boost::bind( callback,
                                      was_success, cname_count ) );
    }
}

// (created using vim -- the world's best text editor)

